package ass.excercises.cvi7.demo1.java8;

import java.util.EnumSet;
import java.util.LinkedList;
import java.util.Set;
import java.util.function.BiConsumer;
import java.util.function.BinaryOperator;
import java.util.function.Function;
import java.util.function.Supplier;
import java.util.stream.Collector;
import java.util.stream.Stream;

import org.apache.commons.lang3.Validate;

public class GroupingCollector implements Collector<String, LinkedList<LinkedList<String>>, LinkedList<LinkedList<String>>>{
	
	final protected int groupSize;
	
	public static void main(String[] args) {
		// the goal is to group elements into list
		String[] tokens = {"a", "b", "c", "d", "e", "f", "g"};
		
		LinkedList<LinkedList<String>> grouped = Stream.of(tokens)
		.collect(new GroupingCollector(2));	// collect using custom collector
		
		System.out.println(grouped);
	}

	public GroupingCollector(int groupSize) {
		Validate.exclusiveBetween(0, Integer.MAX_VALUE, groupSize);
		this.groupSize = groupSize;
	}
	
	// creates function that creates accumulator
	@Override
	public Supplier<LinkedList<LinkedList<String>>> supplier() {
		return () -> {
			LinkedList<LinkedList<String>> accumulator = new LinkedList<LinkedList<String>>();
			accumulator.add(new LinkedList<String>());
			return accumulator;
		};
	}
	
	// an accumulator that accumulates values to accumulator
	@Override
	public BiConsumer<LinkedList<LinkedList<String>>, String> accumulator() {
		return (LinkedList<LinkedList<String>> acc, String item) -> { 
			if(acc.peekLast().size() == groupSize) {
				acc.add(new LinkedList<String>());
			}
			acc.peekLast().add(item);
		};
	}

	// combiner that combines two accumulators into one
	@Override
	public BinaryOperator<LinkedList<LinkedList<String>>> combiner() {
		return (LinkedList<LinkedList<String>> acc1, LinkedList<LinkedList<String>> acc2) -> {
			acc1.addAll(acc2);
			return acc1;
		};
	}

	@Override
	public Set<java.util.stream.Collector.Characteristics> characteristics() {
		return EnumSet.of(Characteristics.IDENTITY_FINISH);
	}
	
	@Override
	public Function<LinkedList<LinkedList<String>>, LinkedList<LinkedList<String>>> finisher() {
		// no need to implement this since we are using Characteristics.IDENTITY_FINISH
		throw new UnsupportedOperationException();
	}

}
