package ass.excercises.cvi7.demo1.rxjava;

import io.reactivex.Observable;
import io.reactivex.Single;
import io.reactivex.functions.BiConsumer;

import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.Callable;

class Ref<T> {
	private T value;

	public Ref(T value) {
		this.value = value;
	}

	public T getValue() {
		return value;
	}

	public void setValue(T value) {
		this.value = value;
	}
}

public class GroupingCollectorRx {
	
	public static void main(String[] args) {
		// the goal is to group elements into list
		String[] tokens = {"a", "b", "c", "d", "e", "f", "g"};

		// first way, loads everything into memory!
		Single<LinkedList<LinkedList<String>>> collected = Observable.fromArray(tokens)
		.collect(supplier(), accumulator(2));
		System.out.println(collected.blockingGet());
		System.out.println("===");
		
		// second way ok
		Observable.fromArray(tokens)
		.buffer(2)
		.forEach(System.out::println);
		System.out.println("===");

		// third way - something you could do, if there was no buffer operator
		alternative(tokens);
	}
	

	private static Callable<LinkedList<LinkedList<String>>> supplier() {
		return () -> {
			LinkedList<LinkedList<String>> accumulator = new LinkedList<>();
			accumulator.add(new LinkedList<String>());
			return accumulator;
		};
	}
	
	private static BiConsumer<LinkedList<LinkedList<String>>, String> accumulator(final int groupingSize) {
		return (LinkedList<LinkedList<String>> acc, String item) -> {
			if(acc.peekLast().size() == groupingSize) {
				acc.add(new LinkedList<>());
			}
			acc.peekLast().add(item);
		};
	}

	private static void alternative(String[] tokens) {
		// create new observable
		Observable.create((emmiter) -> {
			// prepare buffer
			// buffer must be in Ref, because only final variables are accessible in lambda
			final Ref<List<String>> refBuffer = new Ref<>(new LinkedList<>());
			
			
			Observable.fromArray(tokens)
			.doOnError(emmiter::onError)	// pass error to outer obs
			.onErrorResumeNext((Throwable t) -> Observable.<String>empty())	// don't do anything on error
			.forEach(item -> {	// iterate observable when items arrive
				// group by two items
				if(refBuffer.getValue().size() == 2) {
					emmiter.onNext(refBuffer.getValue());
					refBuffer.setValue(new LinkedList<>());
				}
				refBuffer.getValue().add(item);
			});
			
			if(!refBuffer.getValue().isEmpty()) {
				emmiter.onNext(refBuffer.getValue());
			}
			emmiter.onComplete();
		})
		.forEach(System.out::println);
	}
}
